;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xffff71599f7d5ad0de7e344608280ade . "./code.cg2js/cg-acache.fasl")
(#xe6944e8e6041350da9f39a57659f8b4b . "./code.cg2js/cg-base.fasl")
(#x0509a388ba77a9d4f7157fd509280271 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x3b34770625795e8e94f54ec8b6c6ee4a . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb72846c450301f47f4e4a5d701cf5dab . "./code.cg2js/cg-bitmap-stream.fasl")
(#x682e16306daaa8b5b9e98d4bca2fcd1e . "./code.cg2js/cg-button.fasl")
(#xa74ae10a42b0ef5bb28e21f124d06247 . "./code.cg2js/cg-calendar.fasl")
(#x059a300ccfab5258ef4c8a12ad7cb661 . "./code.cg2js/cg-caret.fasl")
(#x43a7517d0b4ff1632c3fb6792fdc1b41 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xe3eaec73ce69102f642dd48c9aa2debd . "./code.cg2js/cg-chart-widget.fasl")
(#xab24a1567a9c6ce4033e8c826206d0f3 . "./code.cg2js/cg-check-box.fasl")
(#x452014ab01526e92bea6031e60a82790 . "./code.cg2js/cg-choice-list.fasl")
(#xae0e9913605866192991c93a7b5b1806 . "./code.cg2js/cg-choose-printer.fasl")
(#xf9d268f22f85c9b7662c61c3d5a78532 . "./code.cg2js/cg-class-grid.fasl")
(#xe20cddf2149259029850f05565e4d6a6 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x306ae6651cd15c393d08302dd7d07f61 . "./code.cg2js/cg-class-support.fasl")
(#x99f1ab4d926d8b13eff9fb789110455a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xf8558a1d2a35f775fd37474199eea396 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1a07ea46c7c6b6d336ccbc1f61a6d0d1 . "./code.cg2js/cg-clipboard.fasl")
(#x67adfaa0f2b464273fb527645d05921a . "./code.cg2js/cg-color-dialog.fasl")
(#x7adc58d3ee46e57907c761e1682cdf20 . "./code.cg2js/cg-combo-box.fasl")
(#xd11506e26d7865fac19d4a694a6c2992 . "./code.cg2js/cg-common-control.fasl")
(#xd96b30d77fc2911c0984c9d2f67a7d15 . "./code.cg2js/cg-comtab.fasl")
(#x1a7e5fb03152cba8578526b9958219c2 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x28041191de45d5dea118758d64f0957f . "./code.cg2js/cg-curve.fasl")
(#xb260d695634f9c403d8ebb39989e6479 . "./code.cg2js/cg-dde-utils.fasl")
(#x5719a48115d49a779fc72911916baaa1 . "./code.cg2js/cg-dev.fasl")
(#xc7da45cf9198807a79995fa412f0e7da . "./code.cg2js/cg-dialog-item.fasl")
(#x70ecf53c3ad4de64f65d83802649777b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe4081b6aeb80d621dbc606890201b0f0 . "./code.cg2js/cg-directory-dialog.fasl")
(#x31fb34552e6922960bcf8e7ca584f5d2 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x1a17c1a3f932f9992e3429cad743b8c2 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x907bca0e3ab616444660bc5cd78b1283 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa1f3d89ea4debe5c5e8f83a1cd6273de . "./code.cg2js/cg-drawable.fasl")
(#x9c6667dd1ba54a3aa3b8a4a812c882b9 . "./code.cg2js/cg-dropping-outline.fasl")
(#xaf68482845239d50edb812dda40c0b90 . "./code.cg2js/cg-edit-in-place.fasl")
(#x1f6976254130ae04ea24030b66246e58 . "./code.cg2js/cg-editable-text.fasl")
(#x6d461b58b19dfd6c588a18cc1aaf353a . "./code.cg2js/cg-file-dialog.fasl")
(#x530c97869dc93dcc8186b45793dbb10e . "./code.cg2js/cg-fill-texture.fasl")
(#x2a20080a4c0d98b405f3fa94c4d8888b . "./code.cg2js/cg-find-string-dialog.fasl")
(#x0b429ef7cb76c9d10487d1999d202495 . "./code.cg2js/cg-font-dialog.fasl")
(#xca9df869d4957e1b3fc1e86feca87a26 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x602cd7f41308022a759548dc21c9bef1 . "./code.cg2js/cg-get-pixmap.fasl")
(#xb359a954069d8cf2bc56250d0ec53f41 . "./code.cg2js/cg-get-position.fasl")
(#xd0b0165bf95c70a27a6c17cc6dc35de9 . "./code.cg2js/cg-graphics-context.fasl")
(#xc19a76c0af30416ec3277bad3b02e610 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1b48752ea1109bc35bbb7d817925f313 . "./code.cg2js/cg-grid-widget.fasl")
(#xe7476f89c45298cf8223f62884adfebf . "./code.cg2js/cg-group-box.fasl")
(#xb495e29d00860745f563ebf6752d430b . "./code.cg2js/cg-header-control.fasl")
(#x3e1ff4178f708d0934bf12349d1aecef . "./code.cg2js/cg-hotspot.fasl")
(#x39bf7f1e5aca33f004b895fcc2db35cb . "./code.cg2js/cg-html-dialog.fasl")
(#x0c35425e5be9824cf1aa3b0f2c214bb4 . "./code.cg2js/cg-html-widget.fasl")
(#x611a10eb97e5fb015c05ef989c79d976 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xaa8526e40bc6985f206fe178d267fac5 . "./code.cg2js/cg-icon.fasl")
(#xa9f8dbd23fc9663a3b5404f4f9a30fe5 . "./code.cg2js/cg-ie.fasl")
(#xa2956cf464e5ccbe9044bc1afb5e35cb . "./code.cg2js/cg-intersect-posbox.fasl")
(#x527a3709a6d38c1605df29d2bfe4884a . "./code.cg2js/cg-intersect.fasl")
(#x7de7ebc300bcae919df4c94bb5382459 . "./code.cg2js/cg-item-list.fasl")
(#x6f4e01e39340ec0ff5c3636c272c48d1 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4ad6d3a371a241924f01a95c2c414255 . "./code.cg2js/cg-lamp.fasl")
(#x03f6179897569b3b1e0d10bf6e5ba0f9 . "./code.cg2js/cg-layout.fasl")
(#x7f30dc31a36689e1078117d5b8a6f119 . "./code.cg2js/cg-lettered-menu.fasl")
(#x0b841bdd9715b7d3e35be0e43be65f4c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x02d540fc3b8c4d449bcd97433b9b336a . "./code.cg2js/cg-lisp-text.fasl")
(#x73b19ff758ce2d09c2bec1c5dc465a04 . "./code.cg2js/cg-lisp-widget.fasl")
(#x08713fce37d26c93a660fa6177faea83 . "./code.cg2js/cg-list-view.fasl")
(#xb12710264191d09192c20863d4e4d740 . "./code.cg2js/cg-mci.fasl")
(#x946955b6be3ae7b4138bf5ab1104d836 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xadfd719f46d289233a24d03e9042e031 . "./code.cg2js/cg-menu.fasl")
(#x662a69230417dee740036badec2053ac . "./code.cg2js/cg-message-dialog.fasl")
(#x81ebbd26eb6af99fb26ee5aa149e60de . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x7acfdb8a8f4ed71ce5e663befd360242 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x2dc5e21d71ffc1fef67b437fecd4552d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x5c013c17ce05cb2f26a0f9f5df3abf08 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x557b94519d03d134ab0fdc431622edc0 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x33c450341bdb648d1d45e5fe9a990c2f . "./code.cg2js/cg-nodes.fasl")
(#xa68b06b085626a38d70e2fc62b3b5d19 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xbf2045c2f31a0c14aae25fd8bf4ed106 . "./code.cg2js/cg-object-editor.fasl")
(#xb3ee5a484540ae8ca8043191c762e311 . "./code.cg2js/cg-ocx.fasl")
(#x522913eccc2939cddeaec97f1b451611 . "./code.cg2js/cg-os-widget.fasl")
(#x329bff3ead8225f3e5b3744c8f6aaaff . "./code.cg2js/cg-os-window.fasl")
(#xa27a5a17e6bceec062a675c2792e06e6 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9f4dd5236afcff0d4e0d3298fc27e601 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xeb0717295388a892c5b4033946f255b6 . "./code.cg2js/cg-outline.fasl")
(#xf58f3e18fe9a424f99365413ec8fc864 . "./code.cg2js/cg-palette.fasl")
(#xf19cbaae9bce5430c24478049a37249a . "./code.cg2js/cg-paren-matching.fasl")
(#xcaba1d5e0ad077ceca738739f1631906 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x71d8bbe41c08d4dcd36f89b0be94f486 . "./code.cg2js/cg-picture-widget.fasl")
(#xab08fdee66bc99460fa91c1d3503a11d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xc197bfbeae355a1d3c3817931b378c4e . "./code.cg2js/cg-pixmap-printing.fasl")
(#x08fe6efb4d9a604cb8579a69d93c9aab . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4433965f79c095142e476e29e6f77e29 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x2801855d7e6d2d4402efbff4466e6edd . "./code.cg2js/cg-pixmap.fasl")
(#x6dadb3cb59c8d5f95734c39ed8b4aa3e . "./code.cg2js/cg-plot-widget.fasl")
(#xe37ca7e3fa7270f451945cd676823fb6 . "./code.cg2js/cg-printing.fasl")
(#x03c9c1210a10e80e10f75c1ba7918fa8 . "./code.cg2js/cg-progress-indicator.fasl")
(#x9b222c42b6465cc63512db0e95da2fd6 . "./code.cg2js/cg-project-window.fasl")
(#x2b05686c1876ca95efba630dc05bba60 . "./code.cg2js/cg-property.fasl")
(#xcf6050f390b1d2bfb3edf7990e0e4b01 . "./code.cg2js/cg-radio-button.fasl")
(#x596058a10224105dfb32638052f48d35 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x43aaf6095083b958562a5d00a72be9ec . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x293d51693ad121257f069b199e92cb25 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x4cde781418226f730d2eee2b29d4e9f6 . "./code.cg2js/cg-rich-edit.fasl")
(#x7647849c6955076944279cec5ed98292 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x7da205209eb774de5d9f2a262c033e94 . "./code.cg2js/cg-scaling-stream.fasl")
(#xf6b4f2f1cde8b21189cb7f8e5d14367b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xdaa5e0127086743766ae5111d15d5c93 . "./code.cg2js/cg-scroll-bar.fasl")
(#xabe04eb7203b71bcfe4fe93090172b32 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xf5364a1f1ea6afbd5bdf5eda4f539b2f . "./code.cg2js/cg-selected-object.fasl")
(#x234e3076b67d1ad51252e9d3f7a66b7a . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb414d73d4945da1cfeb0bca041dce585 . "./code.cg2js/cg-split-bar.fasl")
(#xcd9dc6e34f9ec4095303e3a94030c4f4 . "./code.cg2js/cg-static-text.fasl")
(#xb124e75eeafa1973d89c2a51b01680da . "./code.cg2js/cg-status-bar.fasl")
(#xfcdcbfe92fbf2d0b3b2db31343c0aa4f . "./code.cg2js/cg-string-dialog.fasl")
(#x8eb8d07e7a91515332d8e3ca2c15e6e1 . "./code.cg2js/cg-tab-control.fasl")
(#x42b6d061d0f09a76e7c968092b401390 . "./code.cg2js/cg-template-string.fasl")
(#x598d177a02b21740eef7f2a0bc1cfcd1 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x2eb7bd350f6a68abb93d10e1547e1b44 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe922a5cfcccb353540cd7faedcf24337 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x9fd9ef5bf813821cce0eba4828ad73f8 . "./code.cg2js/cg-text-or-combo.fasl")
(#x3e88094915d89e47039642b8f4468b8e . "./code.cg2js/cg-text-widget.fasl")
(#x3bae5564185dbe745a1dc4acf1102a49 . "./code.cg2js/cg-timer.fasl")
(#x41357a8dea43d41fb44990b65007728c . "./code.cg2js/cg-toggling-widget.fasl")
(#x9dc33f772c22a44b470f27ca8b05d1af . "./code.cg2js/cg-toolbar.fasl")
(#x47c3c5419026121d8aaa11425587d922 . "./code.cg2js/cg-tooltip.fasl")
(#x69ecd17129a0d7d8db68da1d85f33684 . "./code.cg2js/cg-touch.fasl")
(#x128e756ece8420f389af080a007bf855 . "./code.cg2js/cg-trackbar.fasl")
(#xfd9836e0509f6fa43589c288bee8e617 . "./code.cg2js/cg-tray.fasl")
(#xf42b6f3e254c2ea0235f423133704ff3 . "./code.cg2js/cg-up-down-control.fasl")
(#xd5179c6a76596ab5306a44fabb557dfd . "./code.cg2js/cg-utility-dialog.fasl")
(#x2f5e38697b7eebf6790888ba200af95d . "./code.cg2js/cg-web-browser-dde.fasl")
(#x6affba2a4393b69a10a64c429ef1d65b . "./code.cg2js/cg-web-browser.fasl")
(#x7a9da6aef4a6a3e2794b24fc7aed22b7 . "./code.cg2js/cg-wrap-string.fasl")
(#xe38072f9eb06594f2c55daf638040b8c . "./code.cg2js/cg-yes-no-list.fasl")
(#xc6bb5dc817f8f0f40511964294314d29 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6439a1cf9482a2168354e84d16a8521f . "./code.cg2js/cg.fasl")
(#xbc45b8cf781236619836e26cb34c527a . "./code.idejs/ide-acache.fasl")
(#xb0eafc3ff1fb1883757bab05def320ab . "./code.idejs/ide-apropos.fasl")
(#x3de027525ad523b3f951b44db197945f . "./code.idejs/ide-base.fasl")
(#xaf6528a23bc418b6ca5b8cb59f725c8b . "./code.idejs/ide-class-browser.fasl")
(#xe5c5ae4e9b8d20767f6ebe8ff36309f8 . "./code.idejs/ide-class-grapher.fasl")
(#x68570a463ee3d0d7d44243e1bd520d9f . "./code.idejs/ide-clipboard.fasl")
(#x9dafc7e728ee1795db44d1984be40100 . "./code.idejs/ide-dev.fasl")
(#xa99231b3c5a394b54490f0ae2ec3c3b9 . "./code.idejs/ide-editor.fasl")
(#x066be3894339188f868987ca026ed023 . "./code.idejs/ide-find-definitions.fasl")
(#x8899227acdda086b960a65304dbc9cff . "./code.idejs/ide-find-in-files.fasl")
(#xde8ec01c59509f769b9ef554a3c51b55 . "./code.idejs/ide-form.fasl")
(#x3806da00e27a4a3a325cce5e4fe0eaa7 . "./code.idejs/ide-grapher.fasl")
(#x1c624f5ebeaf73ceb9065cb156ecf02d . "./code.idejs/ide-inspector.fasl")
(#x2acd3db269d8f9c4cc34b1f6ebe9450f . "./code.idejs/ide-intro.fasl")
(#xbb8edc9cae3ee4db597d8758bb936b2d . "./code.idejs/ide-keyshort.fasl")
(#x2a9bab6ae03caee990b457bab07ddb46 . "./code.idejs/ide-navigator.fasl")
(#xf2ba017bc44982022bc750b3b22c8a4b . "./code.idejs/ide-package-list.fasl")
(#xdc2a09f04a96598382dd544711a1c170 . "./code.idejs/ide-patch-dialog.fasl")
(#x4733f381407749123d9212f0974e2497 . "./code.idejs/ide-process-browser.fasl")
(#xa8751e4799ff167c5fbf19a49620b887 . "./code.idejs/ide-project.fasl")
(#x8e9e815348d009022a32a3adc537d5b6 . "./code.idejs/ide-trace.fasl")
(#x7b0b8912ffb80590626a468245a68391 . "./code.idejs/ide-window-list.fasl")
(#x022d6f79c4a44dde08daa94b79254130 . "./code.idejs/ide.fasl")
(#xc2912bce53894841a1e9ff8f0fd00d64 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
